@@ -14,7 +14,19 @@ use File::Glob qw(:bsd_glob);
 use POSIX qw(strftime);                                                         # http://www.cplusplus.com/reference/ctime/strftime/
 use Data::Dump qw(dump);
 
-our $VERSION = '2017.109';
+our $VERSION = '2017.112';
+
+&saveToS3('DataTableText') if 0;                                                # Save code
+
+#-------------------------------------------------------------------------------
+# Save source to S3
+#-------------------------------------------------------------------------------
+
+sub saveToS3($)                                                                 # Save to S3:- this will not work, unless you're me, or you happen, to know the key
+ {my ($Z) = @_;                                                                 # File name to be used on S3
+  my $z = $Z.'.zip';
+  print for qx(zip $z $0 && aws s3 cp $z s3://AppaAppsSourceVersions/$z && rm $z);
+ }
 
 #-------------------------------------------------------------------------------
 # Current date and time
@@ -299,8 +311,8 @@ sub loadHashArrayFromLines($)
 
 sub checkKeys($$)
  {my ($test, $permitted) = @_;                                                  # The hash to test, the permitted keys and their meanings
-  my %parms = %$test;
-  delete @parms{$_} for keys %$permitted;
+  my %parms = %$test;                                                           # Copy keys supplied
+  delete $parms{$_} for keys %$permitted;                                       # Remove permitted keys
   return '' unless keys %parms;                                                 # Success - all the keys in the test hash are permitted
 
   confess join "\n",                                                            # Failure - explain what went wrong
@@ -378,6 +390,18 @@ sub nws($)
 
 #0
 #-------------------------------------------------------------------------------
+# Test whether a number is a power of two, return the power if it is else undef
+#-------------------------------------------------------------------------------
+
+sub powerOfTwo($)                                                               # Test whether a number is a power of two, return the power if it is else undef
+ {my ($n) = @_;                                                                 # Number to check
+  for(0..128)
+   {return $_  if 1<<$_ == $n;
+    last       if 1<<$_ >  $n;
+   }
+  undef
+ }
+#-------------------------------------------------------------------------------
 # Documentation
 #-------------------------------------------------------------------------------
 
@@ -473,8 +497,8 @@ extractDocumentation fileList fileSize formatTableBasic
 genLValueArrayMethods genLValueHashMethods genLValueScalarMethods genLValueScalarMethodsWithDefaultValues
 indentString
 loadArrayArrayFromLines loadArrayFromLines  loadHashArrayFromLines loadHashFromLines
-makePath nws
-readBinaryFile readFile timeStamp trim writeBinaryFile writeFile);
+makePath nws powerOfTwo
+readBinaryFile readFile saveToS3 timeStamp trim writeBinaryFile writeFile);
 %EXPORT_TAGS  = (all=>[@EXPORT, @EXPORT_OK]);
 
 1;
